Mapped

Struct Mapped 

pub struct Mapped<Inner, M> {
    pub inner: Inner,
    pub mapper: M,
}
Mapped { inner, mapper } transforms the inner combinator’s value type.

Lossless mappers preserve all format properties including parser soundness and non-malleability, while lossy mappers may introduce malleability.

Fields

inner: Inner

The inner combinator whose values are being transformed.

mapper: M

The mapping between the inner and outer value types.