pub struct Mapped<Inner, M> {
pub inner: Inner,
pub mapper: M,
}Expand description
Mapped { inner, mapper } transforms the inner combinator’s value type.
Lossless mappers preserve all format properties including parser soundness and non-malleability, while lossy mappers may introduce malleability.
Fields
inner: InnerThe inner combinator whose values are being transformed.
mapper: MThe mapping between the inner and outer value types.