pub struct TryMap<Inner, M> {
pub inner: Inner,
pub mapper: M,
}Expand description
TryMap { inner, mapper } is the derived combinator
Mapped { inner: Refined(inner, |v| mapper.wf_in(v)), mapper }.
Parsing fails when the parsed inner value does not satisfy mapper.wf_in.
Serialization maps values back with mapper.spec_map_rev, and consistency
requires both mapper.wf_out(v) and that the reverse-mapped inner value is
consistent and satisfies mapper.wf_in.
Fields
inner: InnerThe inner combinator whose values are being transformed.
mapper: MThe mapping between the inner and outer value types.
Implementations
Sourceimpl<Inner, M: SpecMapper> TryMap<Inner, M>
impl<Inner, M: SpecMapper> TryMap<Inner, M>
Sourcepub open spec fn inner(&self) -> TryMapInner<Inner, M>
pub open spec fn inner(&self) -> TryMapInner<Inner, M>
{
super::Mapped {
inner: Refined(self.inner, |v: M::In| self.mapper.wf_in(v)),
mapper: self.mapper,
}
}Trait Implementations
Sourceimpl<Inner, M> Consistency for TryMap<Inner, M>
impl<Inner, M> Consistency for TryMap<Inner, M>
Sourceopen spec fn consistent(&self, v: Self::Val) -> bool
open spec fn consistent(&self, v: Self::Val) -> bool
{ self.inner().consistent(v) }Sourcetype Val = <M as SpecMapper>::Out
type Val = <M as SpecMapper>::Out
The type of values whose consistency is being checked.
Sourceimpl<Inner, M> EquivSerializers for TryMap<Inner, M>
impl<Inner, M> EquivSerializers for TryMap<Inner, M>
Sourceimpl<Inner, M> EquivSerializersGeneral for TryMap<Inner, M>
impl<Inner, M> EquivSerializersGeneral for TryMap<Inner, M>
Sourceopen spec fn equiv_general_inv(&self) -> bool
open spec fn equiv_general_inv(&self) -> bool
{ self.inner().equiv_general_inv() }Sourceproof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
proof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
Sourceimpl<Inner, M> GoodSerializer for TryMap<Inner, M>
impl<Inner, M> GoodSerializer for TryMap<Inner, M>
Sourceopen spec fn serialize_inv(&self) -> bool
open spec fn serialize_inv(&self) -> bool
{ self.inner().serialize_inv() }Sourceproof fn lemma_serialize_len(&self, v: M::Out)
proof fn lemma_serialize_len(&self, v: M::Out)
Sourceimpl<Inner, M> MinMaxByteLen for TryMap<Inner, M>
impl<Inner, M> MinMaxByteLen for TryMap<Inner, M>
Sourceimpl<Inner, M> NoLookAhead for TryMap<Inner, M>
impl<Inner, M> NoLookAhead for TryMap<Inner, M>
Sourceopen spec fn no_lookahead_inv(&self) -> bool
open spec fn no_lookahead_inv(&self) -> bool
{ self.inner().no_lookahead_inv() }Sourceproof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
proof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
Sourceimpl<Inner, M> NonMalleable for TryMap<Inner, M>
impl<Inner, M> NonMalleable for TryMap<Inner, M>
Sourceopen spec fn nonmal_inv(&self) -> bool
open spec fn nonmal_inv(&self) -> bool
{ self.inner().nonmal_inv() }Sourceproof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
proof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
Sourceimpl<Inner, M> NonTailFmt for TryMap<Inner, M>
impl<Inner, M> NonTailFmt for TryMap<Inner, M>
Sourceopen spec fn serialize_dps_inv(&self) -> bool
open spec fn serialize_dps_inv(&self) -> bool
{ self.inner().serialize_dps_inv() }Sourceproof fn lemma_serialize_dps_prepend(&self, v: M::Out, obuf: Seq<u8>)
proof fn lemma_serialize_dps_prepend(&self, v: M::Out, obuf: Seq<u8>)
Sourceproof fn lemma_serialize_dps_len(&self, v: M::Out, obuf: Seq<u8>)
proof fn lemma_serialize_dps_len(&self, v: M::Out, obuf: Seq<u8>)
Sourceimpl<Inner, M> Productive for TryMap<Inner, M>
impl<Inner, M> Productive for TryMap<Inner, M>
Sourceopen spec fn productive_inv(&self) -> bool
open spec fn productive_inv(&self) -> bool
{ self.inner().productive_inv() }Sourceproof fn lemma_productive(&self, s: Seq<u8>)
proof fn lemma_productive(&self, s: Seq<u8>)
Sourceimpl<Inner, M> SPRoundTripDps for TryMap<Inner, M>
impl<Inner, M> SPRoundTripDps for TryMap<Inner, M>
Sourceopen spec fn unambiguous(&self) -> bool
open spec fn unambiguous(&self) -> bool
{ self.inner().unambiguous() }Sourceproof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
proof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
Sourceimpl<Inner, M> SafeParser for TryMap<Inner, M>
impl<Inner, M> SafeParser for TryMap<Inner, M>
Sourceimpl<Inner, M> SoundParser for TryMap<Inner, M>
impl<Inner, M> SoundParser for TryMap<Inner, M>
Sourceimpl<Inner, M> SpecByteLen for TryMap<Inner, M>
impl<Inner, M> SpecByteLen for TryMap<Inner, M>
Sourceimpl<Inner, M> SpecParser for TryMap<Inner, M>
impl<Inner, M> SpecParser for TryMap<Inner, M>
Sourceimpl<Inner, M> SpecSerializer for TryMap<Inner, M>
impl<Inner, M> SpecSerializer for TryMap<Inner, M>
Sourceopen spec fn spec_serialize(&self, v: M::Out) -> Seq<u8>
open spec fn spec_serialize(&self, v: M::Out) -> Seq<u8>
{ self.inner().spec_serialize(v) }Sourcetype SVal = <M as SpecMapper>::Out
type SVal = <M as SpecMapper>::Out
The type of values to be serialized.
Sourceimpl<Inner, M> SpecSerializerDps for TryMap<Inner, M>
impl<Inner, M> SpecSerializerDps for TryMap<Inner, M>
Sourceimpl<Inner, M> StaticByteLen for TryMap<Inner, M>
impl<Inner, M> StaticByteLen for TryMap<Inner, M>
Sourceopen spec fn static_byte_len() -> nat
open spec fn static_byte_len() -> nat
{ TryMapInner::<Inner, M>::static_byte_len() }Sourceproof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
impl<Inner: Copy, M: Copy> Copy for TryMap<Inner, M>